from fifty_flask.views.generic import TemplateView, SetCookiesMixin
from flask import url_for
from tests.mixins import AppMixin
from unittest import TestCase


class BasicTemplateResponseWithCookies(SetCookiesMixin, TemplateView):
    template_name = 'basic.html'

    def get_response_cookies(self, response):
        cookies = super(BasicTemplateResponseWithCookies, self).get_response_cookies(response)
        cookies.append({
            'key': 'testing',
            'value': 'cookie',
            'max_age': 60 * 60 * 24
        })
        return cookies


class CookiesTests(AppMixin, TestCase):

    def setUp(self):
        super(CookiesTests, self).setUp()
        BasicTemplateResponseWithCookies.add_url_rule(self.app, '/cookies', 'cookies')

    def test_template_view_with_cookies(self):
        """A view that renders a template response.
        """
        response = self.test_app.get(url_for('cookies'))
        cookie_headers = [hdr for hdr in response.headers if hdr[0] == 'Set-Cookie']
        self.assertEqual(len(cookie_headers), 1)
        cookie_str = cookie_headers[0][1]
        self.assertIn('testing=cookie;', cookie_str)
        self.assertIn('Max-Age=86400;', cookie_str)



from __future__ import absolute_import
from flask import current_app
from flask_rq import job


def smart_job(func_or_queue=None, should_noop=False, **smart_job_kwargs):
    noop = lambda: should_noop() if callable(should_noop) else should_noop

    def inner(fn):
        wrapper = job(func_or_queue)
        if not callable(func_or_queue):
            wrapper = wrapper(fn)

        # We're going to replace this with a new function, but internally call the original one.
        real_delay = wrapper.delay

        def delay(*args, **kwargs):
            # Decide if we should run this job at all
            if noop():
                return

            # We need these later on for RQ, but they shouldn't be passed to the function that's
            # being delayed when we're doing a synchronous call
            rq_kwargs = kwargs.pop('rq_kwargs', {})

            # Bypass RQ when appropriate
            if current_app.config.get('RQ_SYNC_SMART_JOB', current_app.debug or current_app.testing):
                return fn(*args, **kwargs)

            # Smart job kwargs first
            # Updated with the fn kwargs
            # Updated with rq_kwargs
            delay_kwargs = {}
            delay_kwargs.update(smart_job_kwargs)
            delay_kwargs.update(kwargs)
            delay_kwargs.update(rq_kwargs)

            return real_delay(*args, **delay_kwargs)

        wrapper.delay = delay

        return wrapper

    # No decorator arguments (use the default behavior)
    if callable(func_or_queue):
        return inner(func_or_queue)

    return inner



from flask import url_for
from fifty_flask.views.generic import GenericView, MimeTypeResponseMixin, \
    TemplateView
from tests.mixins import AppMixin
from unittest import TestCase


class XMLTestView(MimeTypeResponseMixin, GenericView):
    mimetype = 'text/xml'

    def get(self):
        return '<lift>clean and jerk</lift>'


class DefaultMimeTypeTestView(GenericView):

    def get(self):
        return '<html><body>test</body></html>'


class TemplateViewTest(MimeTypeResponseMixin, TemplateView):
    mimetype = 'text/xml'
    template_name = 'mimetypetest.xml'

    def get_context_data(self, **context):
        context = super(TemplateViewTest, self).get_context_data(**context)
        context['message'] = 'Well Hello'
        return context


class MimeTypeResponseTests(AppMixin, TestCase):

    def setUp(self):
        super(MimeTypeResponseTests, self).setUp()
        XMLTestView.add_url_rule(self.app, '/xml/', 'test_xml_mimetype')
        DefaultMimeTypeTestView.add_url_rule(self.app, '/html/', 'test_html_mimetype')
        TemplateViewTest.add_url_rule(self.app, '/template/', 'test_template')
        self.xml_rv = self.test_app.get(url_for('test_xml_mimetype'))
        self.html_rv = self.test_app.get(url_for('test_html_mimetype'))
        self.template_rv = self.test_app.get(url_for('test_template'))


    def test_xml_mimetype(self):
        self.assertEqual(self.xml_rv.status_code, 200)
        self.assertEqual(self.xml_rv.mimetype, 'text/xml')
        self.assertEqual(self.xml_rv.content_type, 'text/xml; charset=utf-8')

    def test_default_mimetype(self):
        self.assertEqual(self.html_rv.status_code, 200)
        self.assertEqual(self.html_rv.mimetype, 'text/html')
        self.assertEqual(self.html_rv.content_type, 'text/html; charset=utf-8')

    def test_template_mimetype(self):
        self.assertEqual(self.template_rv.status_code, 200)
        self.assertEqual(self.template_rv.mimetype, 'text/xml')
        self.assertEqual(self.template_rv.content_type, 'text/xml; charset=utf-8')
        self.assertTrue('Well Hello' in self.template_rv.data)



from fifty_flask.app import Flask


class AppMixin(object):
    def setUp(self):
        self.app = Flask(__name__)
        self.app.testing = True
        self.test_app = self.app.test_client()
        self.ctx = self.app.test_request_context()
        self.ctx.push()

    def tearDown(self):
        self.ctx.pop()



from flask import Blueprint as DefaultBlueprint


class Blueprint(DefaultBlueprint):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('static_folder', 'static')
        kwargs.setdefault('template_folder', 'templates')
        super(Blueprint, self).__init__(*args, **kwargs)



from fifty_flask.views.generic import TemplateResponseMixin, TemplateView
from flask import request, url_for
from tests.mixins import AppMixin
from unittest import TestCase


class NoPjaxView(TemplateView):
    template_name = 'basic.html'


class PjaxView(NoPjaxView):
    pjax_template_name = 'pjax.html'


class OverridePjaxTemplateView(PjaxView):
    def get_context_data(self, **context):
        context = super(OverridePjaxTemplateView, self).get_context_data(**context)
        context['has_context'] = request.args.get('context') == '1'
        return context

    def get_pjax_template_name(self, **context):
        if context.get('has_context'):
            return 'pjax_context.html'
        return 'basic.html'


class PjaxTests(AppMixin, TestCase):
    def setUp(self):
        super(PjaxTests, self).setUp()
        PjaxView.add_url_rule(self.app, '/pjax', 'pjax')
        NoPjaxView.add_url_rule(self.app, '/no-pjax', 'no-pjax')
        OverridePjaxTemplateView.add_url_rule(self.app, '/override-pjax', 'override-pjax')
        self.headers = {'X-PJAX': 1, 'X-Pjax-Container': 'pjax-container'}

    def test_no_pjax_view(self):
        """Send PJAX headers to a view that doesn't explicitly handle PJAX.
        """
        response = self.test_app.get(url_for('no-pjax'), headers=self.headers)
        self.assertEqual(response.data, 'basic template')

    def test_pjax_view(self):
        """Render a PJAX-friendly response.
        """
        response = self.test_app.get(url_for('pjax'), headers=self.headers)
        self.assertEqual(response.data, 'pjax-container\npjax.html')

    def test_override_pjax_template_name(self):
        """Override the default pjax template.
        """
        response = self.test_app.get(url_for('override-pjax'), headers=self.headers)
        self.assertEqual(response.data, 'basic template')

    def test_override_context_pjax_template_name(self):
        """Override the default pjax template by looking at context.
        """
        response = self.test_app.get(url_for('override-pjax', context=1), headers=self.headers)
        self.assertEqual(response.data, 'thar be pjax context here')



from flask import url_for
from fifty_flask.views.generic import CORSMixin, FormView
from tests.forms import PersonForm
from tests.mixins import AppMixin
from unittest import TestCase


class CORSTestFormView(CORSMixin, FormView):
    form_cls = PersonForm
    template_name = 'form.html'

    def get_form_kwargs(self):
        form_kwargs = super(CORSTestFormView, self).get_form_kwargs()
        form_kwargs.setdefault('csrf_enabled', False)
        return form_kwargs


class CORSResponseTests(AppMixin, TestCase):

    def setUp(self):
        super(CORSResponseTests, self).setUp()
        view_func = CORSTestFormView.add_url_rule(self.app, '/cors/', 'test_cors')

    def test_get_cors_view(self):
        with self.test_app:
            response = self.test_app.get(url_for('test_cors'))
            self.assertEquals(response.status_code, 200)
            self.assertEquals('*', response.headers.get('Access-Control-Allow-Origin'))
            self.assertEquals('true', response.headers.get('Access-Control-Allow-Credentials'))
            self.assertEquals('GET, POST, OPTIONS', response.headers.get('Access-Control-Allow-Methods'))

    def test_cors_options_view(self):
        with self.test_app:
            response = self.test_app.options(url_for('test_cors'))
            self.assertEquals(response.status_code, 200)
            self.assertEquals('*', response.headers.get('Access-Control-Allow-Origin'))
            self.assertEquals('true', response.headers.get('Access-Control-Allow-Credentials'))
            self.assertEquals('GET, POST, OPTIONS', response.headers.get('Access-Control-Allow-Methods'))



from fifty_flask.views.generic import GenericView, RedirectView
from flask import request, url_for
from tests.mixins import AppMixin
from unittest import TestCase
from urlparse import urlparse


class RedirectView1(RedirectView):
    pass


class RedirectView2(RedirectView):
    redirect_endpoint = 'lander'


class RedirectView3(RedirectView2):
    def get_context_data(self, **context):
        context = super(RedirectView3, self).get_context_data(**context)
        context['a'] = 'b'
        return context


class LanderView(GenericView):
    def get(self, *args, **kwargs):
        return 'landed'


class RedirectTests(AppMixin, TestCase):
    def setUp(self):
        super(RedirectTests, self).setUp()
        RedirectView1.add_url_rule(self.app, '/redirect1/', 'redirect1')
        RedirectView2.add_url_rule(self.app, '/redirect2/', 'redirect2')
        RedirectView3.add_url_rule(self.app, '/redirect3/', 'redirect3')
        LanderView.add_url_rule(self.app, '/lander/', 'lander')

    def test_redirect1(self):
        """Redirect to self.
        """
        response = self.test_app.get(url_for('redirect1'))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.location, url_for('redirect1', _external=True))

    def test_redirect2(self):
        """Redirect to another route.
        """
        response = self.test_app.get(url_for('redirect2'))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.location, url_for('lander', _external=True))

    def test_redirect3(self):
        """Redirect to another route, passing query string parameters.
        """
        response = self.test_app.get(url_for('redirect3'))
        self.assertEqual(response.status_code, 302)

        parsed_url = urlparse(response.location)
        location = '{scheme}://{host}{path}' \
                .format(scheme=parsed_url.scheme, host=parsed_url.netloc, path=parsed_url.path)
        self.assertEqual(location, url_for('lander', _external=True))
        self.assertEqual(parsed_url.query, 'a=b')



import ast
import os
from flask import Flask as DefaultFlask
from jinja2 import FileSystemBytecodeCache


class Flask(DefaultFlask):
    jinja_key_prefix = 'JINJA'

    def create_jinja_environment(self):
        """Customizes the Jinja environment.
        """
        self.jinja_options = self.get_jinja_options()
        return super(Flask, self).create_jinja_environment()

    def get_jinja_options(self):
        """Get the Jinja config from the app config.
        """
        # start by making the original options mutable
        jinja_options = dict(self.jinja_options)

        # index of the string where the Jinja variable begins
        # example: JINJA_AUTO_RELOAD = True (AUTO_RELOAD is the jinja var we need)
        jinja_key_offset = len(self.jinja_key_prefix) + 1

        # updates the jinja options with new jinja variables from the config
        # lowercases the var name from the config var name to be compatible w/ jinja's lowercase
        # requirement
        jinja_options.update({
            key[jinja_key_offset:].lower(): self.config[key]
            for key in self.config if key.startswith(self.jinja_key_prefix)
            })

        # Enable some sensible defaults when not in debug mode
        if not self.debug:
            jinja_options.setdefault('auto_reload', False)

            if 'bytecode_cache' not in jinja_options:
                directory = self.config.get('FS_BYTECODE_CACHE_DIR')
                jinja_options['bytecode_cache'] = FileSystemBytecodeCache(directory=directory)

        return jinja_options

    def configure(self, config_file_name=None, **config_kwargs):
        """Configures an app with default configuration as well as anything
        specified in config and environment variables.
        """
        # start with project defaults
        self.config.from_pyfile('config_default.py')

        # override with a different default
        if config_file_name:
            self.config.from_pyfile(config_file_name)

        # override with a local config
        self.config.from_pyfile('config_local.py', silent=True)

        # override with config options excplitly passed in the code
        self.config.update(config_kwargs)
        
        # override with the testing configs
        if self.config.get('TESTING'):
            self.config.from_pyfile('config_test_default.py', silent=True)
            self.config.from_pyfile('config_test_local.py', silent=True)

        # override with the environment variables
        env_prefix = self.config.get('ENV_PREFIX')
        if env_prefix:
            env_config = get_env_config(env_prefix)
            self.config.update(env_config)


def get_env_config(env_prefix):
    """Given an environment, this function will look for all variable names
    that start with the package name and update globals() with the key/val
    pairs.
    """
    def cast(value):
        try:
            return ast.literal_eval(value)
        except (ValueError, SyntaxError):
            return value

    env_prefix = env_prefix.upper()

    offset = len(env_prefix) + 1

    return {env_var_name[offset:]: cast(env_var_value)
            for env_var_name, env_var_value in os.environ.iteritems()
            if env_var_name.startswith(env_prefix)}



from flask_wtf import Form
from wtforms import IntegerField, StringField
from wtforms.validators import DataRequired


class PersonForm(Form):
    name = StringField('name', validators=[DataRequired()])
    age = IntegerField('age')



from fifty_flask.views.generic import TemplateResponseMixin, TemplateMixin, TemplateView
from flask import url_for
from tests.mixins import AppMixin
from unittest import TestCase


class BasicTemplateResponse(TemplateResponseMixin):
    template_name = 'basic.html'


class ContextTemplateResponse(TemplateMixin):
    template_name = 'with_context.html'

    def get_context_data(self, **context):
        context = super(ContextTemplateResponse, self).get_context_data(**context)
        context['name'] = 'flask'
        return context


class MyTemplateView(TemplateView):
    template_name = 'basic.html'


class MyTemplateContextView(TemplateView):
    template_name = 'with_context.html'

    def get_context_data(self, **context):
        context = super(MyTemplateContextView, self).get_context_data(**context)
        context['name'] = 'flask'
        return context


class TemplateTests(AppMixin, TestCase):
    def setUp(self):
        super(TemplateTests, self).setUp()
        MyTemplateView.add_url_rule(self.app, '/template', 'template')
        MyTemplateContextView.add_url_rule(self.app, '/template-context', 'template_context')

    def test_render_template(self):
        """Render a template using a mixin.
        """
        btr = BasicTemplateResponse()
        response = btr.render_response()
        self.assertEqual(response, 'basic template')

    def test_render_template_context(self):
        """Render a template with context using a mixin.
        """
        ctr = ContextTemplateResponse()
        context = ctr.get_context_data()
        response = ctr.render_response(**context)
        self.assertEqual(response, 'hello flask')

    def test_template_view(self):
        """A view that renders a template response.
        """
        response = self.test_app.get(url_for('template'))
        self.assertEqual(response.data, 'basic template')

    def test_template_view_context(self):
        """A view that renders a template response with context.
        """
        response = self.test_app.get(url_for('template_context'))
        self.assertEqual(response.data, 'hello flask')



import flask_sqlalchemy
import json
from datetime import datetime
from flask import current_app
from sqlalchemy import Column, DateTime, event
from sqlalchemy.ext.declarative.api import declared_attr
from sqlalchemy.ext.mutable import Mutable, MutableDict as DefaultMutableDict
from sqlalchemy.types import TypeDecorator, TEXT
from werkzeug.local import LocalProxy


_db = LocalProxy(lambda: flask_sqlalchemy.get_state(current_app).db)


def full_commit():
    """A convenience function for executing a transaction with automatic
    rollback when an exception occurs. The exception is re-raised.
    """
    try:
        _db.session.commit()
    except:
        _db.session.rollback()
        raise


class ActiveModel(object):
    """A mixin intended to provide convenient functions for repeated SQLAlchemy
    patterns.
    """
    def save(self, commit=True):
        """Adds the current state of the object to the db session and COMMITs
        the transaction if requested.

        Returns a reference to self for convenience
        """
        return self._transact(_db.session.add, commit)

    def delete(self, commit=True):
        """Deletes the object from the databse and COMMITs the transaction
        if requested.
        """
        return self._transact(_db.session.delete, commit)

    def refresh(self):
        """Immediately re-load attributes from the db.
        """
        _db.session.refresh(self)

    def expire(self):
        """Mark the object so that on next access the attributes are refreshed.
        """
        _db.session.expire(self)

    def expunge(self):
        """Remove the instance from the session.
        """
        _db.session.expunge(self)

    def flush(self):
        """Flush all object changes to the database.
        """
        _db.session.flush()

    def column_changed(self, column):
        """Returns True if a column value changed, False otherwise.
        """
        history = self.column_history(column)
        return history.added or history.deleted

    def column_history(self, column):
        """Return the change details for a column.
        """
        return self._sa_instance_state.get_history(column, True)

    def _transact(self, session_fn, commit):
        """Helper for executing an operation on the session and optionally committing.
        """
        session_fn(self)
        
        if commit:
            full_commit()

        return self


class ImmutableJSONEncodedColumn(TypeDecorator):
    impl = TEXT

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = json.dumps(value, **self.get_dumps_kwargs())
        return value

    def get_dumps_kwargs(self):
        return {'separators': (',', ':')}

    def process_result_value(self, value, dialect):
        if value is not None:
            value = json.loads(value, **self.get_loads_kwargs())
        return value

    def get_loads_kwargs(self):
        return {}


class MutableJSONEncodedColumn(ImmutableJSONEncodedColumn):
    def __init__(self, *args, **kwargs):
        self.null_empty = kwargs.pop('null_empty', False)
        super(MutableJSONEncodedColumn, self).__init__(*args, **kwargs)

    def process_bind_param(self, value, dialect):
        if not value and self.null_empty:
            return None
        return super(MutableJSONEncodedColumn, self).process_bind_param(value, dialect)


class JSONEncodedDict(MutableJSONEncodedColumn):
    pass


class MutableDict(DefaultMutableDict):
    def pop(self, *a):
        value = dict.pop(self, *a)
        self.changed()
        return value

    def popitem(self):
        item = dict.popitem(self)
        self.changed()
        return item


MutableDict.associate_with(JSONEncodedDict)


class JSONEncodedList(MutableJSONEncodedColumn):
    pass


class MutableList(Mutable, list):
    def __setitem__(self, index, value):
        list.__setitem__(self, index, value)
        self.changed()

    def append(self, value):
        list.append(self, value)
        self.changed()

    def extend(self, iterable):
        list.extend(self, iterable)
        self.changed()

    def insert(self, index, value):
        list.insert(self, index, value)
        self.changed()

    def pop(self, index=None):
        value = list.pop(self, index)
        self.changed()
        return value

    def remove(self, value):
        list.remove(self, value)
        self.changed()

    def sort(self, *args, **kwargs):
        list.sort(self, *args, **kwargs)
        self.changed()

    @classmethod
    def coerce(cls, key, value):
        if not isinstance(value, cls):
            if isinstance(value, list):
                return cls(value)
            return Mutable.coerce(key, value)
        else:
            return value

    def __getstate__(self):
        return list(self)

    def __setstate__(self, state):
        self.extend(state)


MutableList.associate_with(JSONEncodedList)


class TimestampedMixin(object):
    """ Mixin to add date created / updated columns to a model.
    """
    @declared_attr
    def date_created(cls):
        return Column(DateTime, default=datetime.utcnow)

    @declared_attr
    def date_updated(cls):
        return Column(DateTime, default=datetime.utcnow)


@event.listens_for(TimestampedMixin, 'before_update', propagate=True)
def timestamped_before_update(mapper, connection, target):
    """ Updates a model with the current time in UTC as the date updated
    """
    target.date_updated = datetime.utcnow()



from fifty_flask.views.generic import FormView
from flask import url_for
from tests.forms import PersonForm
from tests.mixins import AppMixin
from unittest import TestCase


class PersonFormView(FormView):
    form_cls = PersonForm
    template_name = 'form.html'

    def get_form_kwargs(self):
        form_kwargs = super(PersonFormView, self).get_form_kwargs()
        form_kwargs.setdefault('csrf_enabled', False)
        return form_kwargs


class PopulatedPersonFormView(PersonFormView):
    def get_form_obj(self):
        class Person(object):
            name = 'Craig'
        return Person()


class FormViewTests(AppMixin, TestCase):
    def setUp(self):
        super(FormViewTests, self).setUp()
        PersonFormView.add_url_rule(self.app, '/form-view/', 'form_view')
        PopulatedPersonFormView.add_url_rule(self.app, '/populated-form-view/',
                'populated_form_view')

    def test_form_get(self):
        """The response should render an empty form.
        """
        response = self.test_app.get(url_for('form_view'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('name="name" type="text" value=""', response.data)
        self.assertIn('name="age" type="text" value=""', response.data)

    def test_valid_form_post(self):
        """The response should be a redirect, since it's a valid form post.
        """
        data = {
                'name': 'Craig',
                'age': 32
                }

        response = self.test_app.post(url_for('form_view'), data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.location, url_for('form_view', _external=True))

    def test_invalid_form_post(self):
        """The response should include a form validation error, with the fields still populated
        with the posted data.
        """
        data = {
                'age': 32
                }

        response = self.test_app.post(url_for('form_view'), data=data)
        self.assertEqual(response.status_code, 200)
        self.assertIn('This field is required', response.data)
        self.assertIn('name="name" type="text" value=""', response.data)
        self.assertIn('name="age" type="text" value="32"', response.data)

    def test_populated_form_get(self):
        """The response should have a pre-populated text field.
        """
        response = self.test_app.get(url_for('populated_form_view'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('name="name" type="text" value="Craig"', response.data)



from flask import after_this_request, jsonify, redirect, render_template, request, Response, send_file, url_for
from flask.views import MethodView


def url_rule(app, rule, *args, **kwargs):
    """A class decorator for generic views, which provides the same feel of the @app.route
    decorator on normal function views.

    :param cls: A pluggable view to decorate
    :return: The pluggable view
    """
    def decorator(cls):
        add_url_rule = cls.add_url_rule \
                if isinstance(rule, basestring) \
                else cls.add_url_rules

        add_url_rule(app, rule, *args, **kwargs)

        return cls

    return decorator


class GenericView(MethodView):
    """The base generic view class.
    """
    def dispatch_request(self, *args, **kwargs):
        """Sets self.args and self.kwargs from the request for convenience, in
        case there is a context in the code where the data is not directly
        passed in, but you need access to it.
        """
        self.args = args
        self.kwargs = kwargs
        return super(GenericView, self).dispatch_request(*args, **kwargs)

    @classmethod
    def add_url_rule(cls, app, rule, endpoint, view_func=None, **options):
        """Convenience for registering this view on an app or blueprint,
        responding to the provided rule and given the specified endpoint name.

        :param app: A flask app or blueprint
        :param rule: The url pattern to match
        :param endpoint: The name of this route, for use with :func:`~flask.url_for`
        :param view_func: A reference to the previous view func, if matching multiple routes
        :param options: Optional arguments passed directly into :meth:`Flask.add_url_rule` or
                        :meth:`Blueprint.add_url_rule` depending on if `app` is a Flask or
                        Blueprint instance
        """
        if not view_func:
            view_func = cls.as_view(endpoint)

        app.add_url_rule(rule, view_func=view_func, **options)

        return view_func

    @classmethod
    def add_url_rules(cls, app, rules, endpoint, view_func=None, **options):
        """If there needs to be multiple endpoints mapped to a single view, this class-method
        should be used in order to guarantee the view_func is the same. Takes the same arguments as
        :meth:`add_url_rule`, with the only exception being that `rules` is a list of patterns.

        :param rules: A list of url patterns to match
        """
        for rule in rules:
            view_func = cls.add_url_rule(app, rule, endpoint, view_func, **options)

        return view_func


class ResponseMixin(object):
    def render_response(self, **context):
        """Renders a response, optionally with the provided context.

        :param context: An optional dict for use in a rendered response.
        """
        raise NotImplementedError


class ContextMixin(object):
    def get_context_data(self, **context):
        """Constructs and returns a dict for use as context in a rendered response. This method
        should only be used when directly providing context intended to be used in a response.
        There should be no side-effects introduced here.
        
        If you need to provide a class-member set of shared context for use in an inherited mixin
        chain, then you should override :meth:`~GenericView.dispatch_request` and assign that
        shared context before calling the `super` implementation of that method.

        :param context: An optional dict for use in a rendered response.
        :return: A dict
        """
        return context


class RedirectMixin(object):
    redirect_endpoint = None

    def redirect(self, **context):
        """Redirect to another URL with the provided context getting passed to url_for
        """
        url = self.get_redirect_url(**context)
        return redirect(url)

    def get_redirect_url(self, **context):
        """Returns a URL that the view will redirect to by default when the
        form is validated.
        """
        redirect_endpoint = self.get_redirect_endpoint(**context)
        if redirect_endpoint:
            return url_for(redirect_endpoint, **context)

        return request.url

    def get_redirect_endpoint(self, **context):
        """Returns the endpoint to redirect to with the provided context.
        """
        return self.redirect_endpoint


class RedirectView(ContextMixin, RedirectMixin, GenericView):
    def get(self, *args, **kwargs):
        """Adds a generic view for redirecting to another endpoint.
        """
        context = self.get_context_data(**kwargs)
        return self.redirect(**context)


class TemplateResponseMixin(ResponseMixin):
    """A mixin for rendering a template.
    """
    #: The name of the template to render
    template_name = None

    def get_template(self, **context):
        """Gets the template that needs to be rendered.
        """
        return self.template_name

    def render_response(self, **context):
        """Renders a template with the provided context. If a pjax template is
        defined, it will attempt to serve it if the request headers indicated
        that a PJAX response is needed.
        """
        template = self.get_template(**context)
        return render_template(template, **context)


class TemplateMixin(ContextMixin, TemplateResponseMixin):
    """A template response with context.
    """


class PjaxResponseMixin(object):
    """A mixin for rendering a PJAX response.
    """
    pjax_template_name = None

    @property
    def is_pjax_request(self):
        """Returns True if it's a PJAX request, False otherwise.
        """
        return 'X-PJAX' in request.headers

    def get_template(self, **context):
        """Renders a PJAX-friendly response.
        """
        if self.is_pjax_request:
            pjax_template_name = self.get_pjax_template_name(**context)

            if pjax_template_name:
                return pjax_template_name

        return super(PjaxResponseMixin, self).get_template(**context)

    def get_context_data(self, **context):
        """Adds PJAX information to the context, for use in rendered templates.
        """
        context = super(PjaxResponseMixin, self).get_context_data(**context)

        if self.is_pjax_request:
            context['pjax_container'] = request.headers.get('X-Pjax-Container')

            pjax_template_name = self.get_pjax_template_name(**context)
            if pjax_template_name:
                context['pjax_template'] = pjax_template_name

        return context

    def get_pjax_template_name(self, **context):
        """Gets the PJAX-friendly template that could potentially be rendered.
        """
        return self.pjax_template_name


class PjaxTemplateMixin(PjaxResponseMixin, TemplateMixin):
    """A PJAX-friendly template response.
    """


class TemplateView(PjaxTemplateMixin, GenericView):
    """A view that renders a template on a GET request.
    """
    def get(self, *args, **kwargs):
        """Gets the context data and renders a template with it.
        """
        context = self.get_context_data(**kwargs)
        return self.render_response(**context)


class FormMixin(ResponseMixin):
    """A mixin for processing a form.
    """
    #: A form class used to process POST data. Must inherit from flask-wtf form
    form_cls = None

    def process_form(self):
        """Workflow for processing a form, from flask-wtf. If the request method is POST and the
        form is validated, it returns the result of `self.form_valid(form)`. If the form data is
        invalid, or the request method is GET, it returns the result of `self.form_invalid(form)`.
        """
        form = self.get_form()

        if form.validate_on_submit():
            return self.form_valid(form)

        return self.form_invalid(form)

    def get_form(self):
        """Instantiates a new form from the form class and any arguments that
        are defined for its constructor through self.get_form_kwargs()
        """
        form_cls = self.get_form_cls()
        form_kwargs = self.get_form_kwargs()
        return form_cls(**form_kwargs)

    def get_form_cls(self):
        """Default behavior is to return the form_cls defined on this instance.
        """
        return self.form_cls

    def get_form_kwargs(self):
        """Any arguments that need to be passed to the form class constructor
        should be defined here and returned as a dict.
        """
        form_kwargs = {}
        form_obj = self.get_form_obj()
        if form_obj:
            form_kwargs['obj'] = form_obj
        return form_kwargs

    def get_form_obj(self):
        """ The form object to pre-populate the form with.
        """

    def form_valid(self, form, **context):
        """Default behavior on form validation is to simply redirect to the URL
        returned from self.get_success_url()
        """
        context = self.get_context_data(form=form, **context)
        return self.render_response(**context)

    def form_invalid(self, form, **context):
        """If a form is not valid, the default behavior is to render the
        template and including the form as context.
        """
        context = self.get_context_data(form=form, **context)
        return self.render_response(**context)


class ProcessFormView(FormMixin, GenericView):
    def post(self, *args, **kwargs):
        """Creates a form and validates it. If validation is successful, it
        will call self.form_valid(form). Otherwise, it will call
        self.form_invalid(form). The default behavior for each of these methods
        is documented in the FormMixin class.
        """
        return self.process_form()

        
class FormView(PjaxTemplateMixin, RedirectMixin, ProcessFormView):
    def get(self, *args, **kwargs):
        """Creates a form and includes it as context to a template that will be
        rendered.
        """
        form = self.get_form()
        context = self.get_context_data(form=form, **kwargs)
        return self.render_response(**context)

    def form_valid(self, form, **context):
        """Default behavior on form validation is to simply redirect to the URL
        returned from self.get_success_url()
        """
        return self.redirect(**context)


class JsonResponseMixin(ResponseMixin):
    def render_response(self, **context):
        """Returns a JSON response with the provided context.
        """
        return jsonify(**context)


class JsonMixin(ContextMixin, JsonResponseMixin):
    """A json response with context.
    """


class AjaxView(JsonMixin, GenericView):
    def get(self, *args, **kwargs):
        """Sends a jsonified response from a GET request.
        """
        context = self.get_context_data()
        return self.render_response(**context)


class AjaxFormView(JsonMixin, ProcessFormView):
    """Processes an AJAX form.
    """
    def form_invalid(self, form, **context):
        context.setdefault('errors', form.errors)
        context = self.get_context_data(form=form, **context)

        response = self.render_response(**context)
        response.status_code = 400
        return response

    def get_context_data(self, form=None, **context):
        return super(AjaxFormView, self).get_context_data(**context)


class CustomResponseMixin(object):
    response_cls = Response

    def dispatch_request(self, *args, **kwargs):
        """Returns a custom Response object, using the result of the response from the dispatched
        request as well as any custom response parameters.
        """
        response = super(CustomResponseMixin, self).dispatch_request(*args, **kwargs)
        response_cls = self.get_response_cls()
        response_kwargs = self.get_response_kwargs(response)
        return response_cls(response=response, **response_kwargs)

    def get_response_cls(self):
        """Returns a Response class.
        """
        return self.response_cls

    def get_response_kwargs(self, response):
        """Returns a dict of kwargs passed into the constructor of the response class returned from
        self.get_response_cls()
        """
        return {}


class MimeTypeMixin(object):
    mimetype = None

    def get_mimetype(self):
        """Returns a mimetype.
        """
        return self.mimetype


class MimeTypeResponseMixin(MimeTypeMixin, CustomResponseMixin):
    def get_response_kwargs(self, response):
        """If a mimetype is specified, include it when instantiating the Response instance.
        """
        response_kwargs = super(MimeTypeResponseMixin, self).get_response_kwargs(response)
        response_kwargs.setdefault('mimetype', self.get_mimetype())
        return response_kwargs


class SendFileMixin(MimeTypeMixin):
    #: A file pointer of path to a file
    filename_or_fp = None

    #: By default, use Content-Disposition: attachment
    as_attachment = True

    #: The filename of the attachment
    attachment_filename = None

    def get_filename_or_fp(self):
        """Returns a filename or file pointer.
        """
        return self.filename_or_fp

    def get_as_attachment(self):
        """Returns True if the file should be served as an attachment, False otherwise.
        """
        return self.as_attachment

    def get_attachment_filename(self):
        """Returns the filename for the attachment.
        """
        return self.attachment_filename

    def get_sendfile_kwargs(self):
        """Returns a dict of arguments to be passed to :func:`~flask.send_file`.
        """
        return {}

    def send_file(self):
        """Returns the result of :func:`~flask.send_file`
        """
        send_file_kwargs = self.get_sendfile_kwargs()
        send_file_kwargs.setdefault('filename_or_fp', self.get_filename_or_fp())
        send_file_kwargs.setdefault('mimetype', self.get_mimetype())
        send_file_kwargs.setdefault('as_attachment', self.get_as_attachment())
        send_file_kwargs.setdefault('attachment_filename', self.get_attachment_filename())
        return send_file(**send_file_kwargs)


class SendFileView(SendFileMixin, GenericView):
    def get(self, *args, **kwargs):
        """Returns the response from :func:`~flask.send_file`.
        """
        return self.send_file()


class SetCookiesMixin(object):

    def dispatch_request(self, *args, **kwargs):
        after_this_request(self.add_response_cookies)
        return super(SetCookiesMixin, self).dispatch_request(*args, **kwargs)

    def add_response_cookies(self, response):
        """Sets cookies on the response object.
        """
        for cookie_context in self.get_response_cookies(response):
            response.set_cookie(**cookie_context)
        return response

    def get_response_cookies(self, response):
        return []


class JsonFormMixin(object):
    """ Mixin to process forms using the from_json api. Please note: this mixin requires wtforms-json to be installed.
    """

    skip_unknown_keys = True
    form_json = None
    csrf_enabled = False

    def get_form(self):
        form_cls = self.get_form_cls()
        form_kwargs = self.get_form_kwargs()
        return form_cls.from_json(**form_kwargs) if 'formdata' in form_kwargs else form_cls(**form_kwargs)

    def get_form_kwargs(self):
        form_kwargs = super(JsonFormMixin, self).get_form_kwargs()
        self.form_json = self.get_form_json()
        if 'formdata' not in form_kwargs and self.form_json:
            form_kwargs['formdata'] = self.form_json
            form_kwargs.setdefault('skip_unknown_keys', self.skip_unknown_keys)
            form_kwargs.setdefault('csrf_enabled', self.get_csrf_enabled())
        return form_kwargs

    def get_form_json(self):
        return request.get_json()

    def get_csrf_enabled(self):
        return self.csrf_enabled


class JsonFormView(JsonFormMixin, AjaxFormView):
    """ View to process forms using the from_json api.
    """
    def get_context_data(self, form=None, **context):
        context = super(JsonFormView, self).get_context_data(form=form, **context)
        context.setdefault('status', 'error' if context.get('errors') else 'success')
        return context


class CORSMixin(object):
    """ Mixin to add CORS response headers.
    """

    cors_allow_origin = '*'
    cors_allow_credentials = True
    cors_allow_methods = None

    @classmethod
    def as_view(cls, name, *class_args, **class_kwargs):
        view_func = super(CORSMixin, cls).as_view(name, *class_args, **class_kwargs)
        view_func.provide_automatic_options = False
        if view_func.methods and 'OPTIONS' not in view_func.methods:
            view_func.methods.append('OPTIONS')
        return view_func
    
    def dispatch_request(self, *args, **kwargs):
        after_this_request(self.add_cors_response_headers)
        return super(CORSMixin, self).dispatch_request(*args, **kwargs)

    def options(self, *args, **kwargs):
        return ''

    def add_cors_response_headers(self, response):
        response.headers['Access-Control-Allow-Origin'] = self.get_cors_allow_origin()
        response.headers['Access-Control-Allow-Credentials'] = 'true' if self.get_cors_allow_credentials() else 'false'
        response.headers['Access-Control-Allow-Methods'] = ', '.join(self.get_cors_allow_methods())
        return response

    def get_cors_allow_origin(self):
        return self.cors_allow_origin

    def get_cors_allow_credentials(self):
        return self.cors_allow_credentials

    def get_cors_allow_methods(self):
        return self.cors_allow_methods or self.methods



import os
import time
from unittest import TestCase
from .mixins import FiftyConfigMixin, InstanceRelativeMixin


class NoDefaultConfig(FiftyConfigMixin, TestCase):
    def test(self):
        """The app should use the defaults specified in flask since there is no default config.
        """
        self.assertFalse(self.app.debug)
        self.assertFalse(self.app.testing)


class DefaultConfig(InstanceRelativeMixin, TestCase):
    instance_relative_path = 'default'

    def test_debug(self):
        """The app should still be in debug mode by default.
        """
        self.assertTrue(self.app.debug)

    def test_secret(self):
        """The secret key should exist in the default config.
        """
        self.assertEqual(self.app.config['SECRET_KEY'], 'abcdefg')

    def test_missing_value(self):
        """There should be no timezone specified in the default config.
        """
        self.assertNotIn('TZ', self.app.config)


class LocalConfig(InstanceRelativeMixin, TestCase):
    instance_relative_path = 'local_override'

    def test_override(self):
        """The local config should be able to change the values of the default settings.
        """
        self.assertFalse(self.app.debug)
        self.assertEqual(self.app.config['SECRET_KEY'], 'zyxwvut')

    def test_add_config_value(self):
        """The local config should be able to add a new config variable.
        """
        self.assertTrue(self.app.config['TESTING'])

    def test_python(self):
        """The local config should be able to execute python code.
        """
        self.assertEqual(self.app.config['TZ'], time.tzname)


class ExternalConfig(InstanceRelativeMixin, TestCase):
    instance_relative_path = 'local_override'

    def test_override(self):
        """The external config should change the default secret key.
        """
        self.assertEqual(self.app.config['SECRET_KEY'], 'zyxwvut')

    def test_add_config_value(self):
        """The external config should add a new config variable.
        """
        self.assertEqual(self.app.config['TOTALLY_RANDOM'], 1234)

    def get_app_config_kwargs(self):
        return dict(config_file_name='random_config.py')


class EnvConfig(InstanceRelativeMixin, TestCase):
    instance_relative_path = 'local_override'

    def setUp(self):
        """Add an environment variable.
        """
        os.environ['FIFTY_ENV_STR'] = 'hello'
        os.environ['FIFTY_ENV_BOOL'] = 'True'
        super(EnvConfig, self).setUp()

    def test_env_var(self):
        """The app should be able to load config data from the environment.
        """
        self.assertEqual(self.app.config['ENV_STR'], 'hello')
        self.assertTrue(self.app.config['ENV_BOOL'])



from fifty_flask.views.generic import GenericView, url_rule
from flask import url_for
from tests.mixins import AppMixin
from unittest import TestCase


class IndexView(GenericView):
    def get(self, *args, **kwargs):
        return 'get'

    def post(self, *args, **kwargs):
        return 'post'


class ArgsView(GenericView):
    def get(self, *args, **kwargs):
        assert args == self.args, '*args is different'
        assert kwargs == self.kwargs, '**kwargs is different'
        assert 'test' in kwargs
        return ''


class GenericViewTests(AppMixin, TestCase):
    def setUp(self):
        super(GenericViewTests, self).setUp()
        IndexView.add_url_rule(self.app, '/', 'index')
        ArgsView.add_url_rule(self.app, '/args/<test>', 'args')

    def test_get(self):
        """View responds to a GET.
        """
        response = self.test_app.get(url_for('index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, 'get')

    def test_post(self):
        """View responds to a POST.
        """
        response = self.test_app.post(url_for('index'), data={})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, 'post')

    def test_missing_method(self):
        """View does not respond to a disallowed request method.
        """
        response = self.test_app.delete(url_for('index'), data={})
        self.assertEqual(response.status_code, 405)

    def test_args_kwargs(self):
        """*args and **kwargs are available anywhere in the view via self.
        """
        self.test_app.get(url_for('args', test='test'))

    def test_url_rule_single_route(self):
        """View class can be decorated with a single route.
        """
        route = '/single-route-view/'
        route_name = 'single_route_view'
        route_response = 'single route ok'

        @url_rule(self.app, route, route_name)
        class SingleRouteView(GenericView):
            def get(self, *args, **kwargs):
                return route_response

        response = self.test_app.get(route)
        self.assertEqual(response.data, route_response)
        self.assertEqual(url_for(route_name), route)

    def test_url_rule_multiple_routes(self):
        """View class can be decorated with multiple routes.
        """
        routes = ['/route1-view/', '/route2-view/']
        route_name = 'multiple_route_view'
        route_response = 'multiple routes ok'

        @url_rule(self.app, routes, route_name)
        class MultipleRouteView(GenericView):
            def get(self, *args, **kwargs):
                return route_response

        for route in routes:
            response = self.test_app.get(route)
            self.assertEqual(response.data, route_response)

        self.assertEqual(url_for(route_name), routes[0])



from fifty_flask.views.generic import ResponseMixin, ContextMixin
from tests.mixins import AppMixin
from unittest import TestCase


class StrResponse(ResponseMixin):
    def render_response(self, **context):
        return 'hello there'


class InheritedContext(ContextMixin):
    def get_context_data(self, **context):
        context = super(InheritedContext, self).get_context_data(**context)
        context['base'] = True
        return context


class OverrideContext(InheritedContext):
    def get_context_data(self, **context):
        context = super(OverrideContext, self).get_context_data(**context)
        context['base'] = False
        return context


class UpdateContext(InheritedContext):
    def get_context_data(self, **context):
        context = super(UpdateContext, self).get_context_data(**context)
        context['updated'] = True
        return context


class RemoveContext(UpdateContext):
    def get_context_data(self, **context):
        context = super(RemoveContext, self).get_context_data(**context)
        del context['updated']
        return context


class ReplaceContext(InheritedContext):
    def get_context_data(self, **context):
        return {'replaced': True}


class ResponseTests(TestCase):
    pass


class BaseContextTests(TestCase):
    def test_base_context(self):
        """Default context is an empty dict.
        """
        context = ContextMixin().get_context_data()
        self.assertEqual(context, {})

    def test_explicit_context(self):
        """Passing context directly into the mixin should return that same context.
        """
        d = {'a': 'b'}
        context = ContextMixin().get_context_data(**d)
        self.assertEqual(context, d)

    def test_inherit_context(self):
        """Inherit from base context and add to it.
        """
        context = InheritedContext().get_context_data()
        self.assertEqual(context, {'base': True})

    def test_override_context(self):
        """Inherit from base context and change an existing value.
        """
        context = OverrideContext().get_context_data()
        self.assertEqual(context, {'base': False})

    def test_update_context(self):
        """Inherit from base context and add a new value.
        """
        context = UpdateContext().get_context_data()
        self.assertEqual(context, {'base': True, 'updated': True})

    def test_remove_context(self):
        """Inherit from update context and remove a value.
        """
        context = RemoveContext().get_context_data()
        self.assertEqual(context, {'base': True})

    def test_replace_context(self):
        """Inherit from base context, but ignore it completely and return new context.
        """
        context = ReplaceContext().get_context_data()
        self.assertEqual(context, {'replaced': True})



import os
from StringIO import StringIO
from fifty_flask.views.generic import SendFileView
from flask import current_app, url_for
from tests.mixins import AppMixin
from unittest import TestCase


class HtmlMimetypeMixin(object):
    mimetype = 'text/html'


class InlineFilenameView(SendFileView):
    as_attachment = False
    filename_or_fp = 'files/file'


class InlineFPView(SendFileView):
    as_attachment = False

    def get_filename_or_fp(self):
        return open(os.path.join(current_app.root_path, 'files/file'))


class InlineMimetypeView(HtmlMimetypeMixin, InlineFilenameView):
    pass


class AttachmentFilenameView(SendFileView):
    as_attachment = True
    filename_or_fp = 'files/file'


class AttachmentFilenameOverrideView(AttachmentFilenameView):
    attachment_filename = 'myfile'


class AttachmentMimetypeView(HtmlMimetypeMixin, AttachmentFilenameView):
    pass


class StringIOView(SendFileView):
    attachment_filename = 'file'

    def get_filename_or_fp(self):
        with open(os.path.join(current_app.root_path, 'files/file')) as f:
            return StringIO(f.read())


class SendFileTests(AppMixin, TestCase):
    def setUp(self):
        super(SendFileTests, self).setUp()
        self.file_content = 'file content'
        InlineFilenameView.add_url_rule(self.app, '/inline-filename/', 'inline_filename')
        InlineFPView.add_url_rule(self.app, '/inline-fp/', 'inline_fp')
        InlineMimetypeView.add_url_rule(self.app, '/inline-mimetype/', 'inline_mimetype')
        AttachmentFilenameView.add_url_rule(self.app, '/attachment-filename/', 'attachment_filename')
        AttachmentFilenameOverrideView.add_url_rule(self.app, '/attachment-filename-override/',
                'attachment_filename_override')
        StringIOView.add_url_rule(self.app, '/sio/', 'sio')
        AttachmentMimetypeView.add_url_rule(self.app, '/attachment-mimetype/',
                'attachment_mimetype')

    def test_inline_filename(self):
        """Serve file inline.
        """
        response = self.test_app.get(url_for('inline_filename'))
        self.assertIsInline(response)
        self.assertEqual(response.data.strip(), self.file_content)

    def test_inline_fp(self):
        """Serve file from file pointer inline.
        """
        response = self.test_app.get(url_for('inline_fp'))
        self.assertIsInline(response)
        self.assertEqual(response.data.strip(), self.file_content)

    def test_inline_mimetype(self):
        """Serve file inline with custom mimetype.
        """
        response = self.test_app.get(url_for('inline_mimetype'))
        self.assertIsInline(response)
        self.assertEqual(response.content_type, 'text/html; charset=utf-8')
        self.assertEqual(response.data.strip(), self.file_content)

    def test_attachment(self):
        """Serve file as attachemnt.
        """
        response = self.test_app.get(url_for('attachment_filename'))
        self.assertIsAttachment(response)
        self.assertEqual(response.data.strip(), self.file_content)

    def test_attachment_override(self):
        """Serve file as attachment with custom name.
        """
        response = self.test_app.get(url_for('attachment_filename_override'))
        self.assertHasFilename(response, 'myfile')
        self.assertEqual(response.data.strip(), self.file_content)

    def test_attachment_mimetype(self):
        """Serve file as attachment with custom mimetype.
        """
        response = self.test_app.get(url_for('attachment_mimetype'))
        self.assertIsAttachment(response)
        self.assertEqual(response.content_type, 'text/html; charset=utf-8')
        self.assertEqual(response.data.strip(), self.file_content)

    def test_sio(self):
        """Serve string as attachment using StringIO.
        """
        response = self.test_app.get(url_for('sio'))
        self.assertIsAttachment(response)
        self.assertEqual(response.data.strip(), self.file_content)

    def assertIsAttachment(self, response):
        """Asserts the response has an attachment.
        """
        content_disposition = response.headers.get('Content-Disposition')
        self.assertIsNotNone(content_disposition)
        self.assertTrue(content_disposition.startswith('attachment;'))

    def assertIsInline(self, response):
        """Asserts the response is not served as an attachment.
        """
        self.assertIsNone(response.headers.get('Content-Disposition'))

    def assertHasFilename(self, response, filename):
        """Asserts the response has a specific filename.
        """
        self.assertIsAttachment(response)
        content_disposition = response.headers.get('Content-Disposition')
        self.assertTrue(content_disposition.endswith('filename={}'.format(filename)))



from setuptools import find_packages, setup

setup(
    name='Fifty-Flask',
    version='1.2.0',
    url='https://bitbucket.org/50onred/fifty/overview',
    license='BSD',
    author='Craig Slusher',
    author_email='craig@50onred.com',
    description='Flask enhancements.',
    long_description=__doc__,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'flask>=0.11',
    ],
    tests_require=[
        'nose'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



import os
from fifty_flask.app import Flask

class FiftyConfigMixin(object):
    instance_relative_config = False
    instance_relative_path = None

    def setUp(self):
        instance_path = os.path.abspath(os.path.join(os.path.dirname(__file__),
            self.instance_relative_path)) \
                    if self.instance_relative_path \
                    else None

        self.app = Flask(__name__, instance_path=instance_path,
                instance_relative_config=self.instance_relative_config)

class InstanceRelativeMixin(FiftyConfigMixin):
    instance_relative_config = True

    def setUp(self):
        super(InstanceRelativeMixin, self).setUp()
        self.app.configure(**self.get_app_config_kwargs())

    def get_app_config_kwargs(self):
        return {}
